@@ -69,11 +69,13 @@ typedef struct {
 	gboolean        hidden;
 	gboolean        not_null; /* TRUE if @entry's contents can't be NULL */
 	gboolean        can_expand; /* tells if @entry can expand */
+	gboolean        forward_param_updates; /* forward them to the GdauiDataEntry widgets ? */
 
 	gulong          entry_shown_id; /* signal ID */
 	gulong          label_shown_id; /* signal ID */
 
 	gulong          entry_contents_modified_id; /* signal ID */
+	gulong          entry_expand_changed_id; /* signal ID */
 	gulong          entry_contents_activated_id; /* signal ID */
 
 	GdaHolder      *single_param;
@@ -98,6 +100,7 @@ static void pack_entries_in_table (GdauiBasicForm *form);
 static void pack_entries_in_xml_layout (GdauiBasicForm *form, xmlNodePtr form_node);
 static void unpack_entries (GdauiBasicForm *form);
 static void destroy_entries (GdauiBasicForm *form);
+static gchar *create_text_label_for_sentry (SingleEntry *sentry, gchar **out_title);
 
 static void gdaui_basic_form_show_entry_actions (GdauiBasicForm *form, gboolean show_actions);
 static void gdaui_basic_form_set_entries_auto_default (GdauiBasicForm *form, gboolean auto_default);
@@ -106,8 +109,11 @@ static void get_rid_of_set (GdaSet *paramlist, GdauiBasicForm *form);
 static void paramlist_public_data_changed_cb (GdauiSet *paramlist, GdauiBasicForm *form);
 static void paramlist_param_attr_changed_cb (GdaSet *paramlist, GdaHolder *param,
 					     const gchar *att_name, const GValue *att_value, GdauiBasicForm *form);
+static void paramlist_holder_type_set_cb (GdaSet *paramlist, GdaHolder *param,
+					  GdauiBasicForm *form);
 
 static void entry_contents_modified (GdauiDataEntry *entry, SingleEntry *sentry);
+static void entry_expand_changed_cb (GdauiDataEntry *entry, SingleEntry *sentry);
 static void entry_contents_activated (GdauiDataEntry *entry, GdauiBasicForm *form);
 static void parameter_changed_cb (GdaHolder *param, SingleEntry *sentry);
 
@@ -127,7 +133,7 @@ enum {
 	PROP_HEADERS_SENSITIVE,
 	PROP_SHOW_ACTIONS,
 	PROP_ENTRIES_AUTO_DEFAULT,
-	PROP_CAN_EXPAND
+	PROP_CAN_VEXPAND
 };
 
 typedef struct {
@@ -151,7 +157,6 @@ struct _GdauiBasicFormPriv
 	GtkWidget              *top_container;
 
 	gboolean                headers_sensitive;
-	gboolean                forward_param_updates; /* forward them to the GdauiDataEntry widgets ? */
 	gboolean                show_actions;
 	gboolean                entries_auto_default;
 
@@ -273,9 +278,9 @@ gdaui_basic_form_class_init (GdauiBasicFormClass * class)
 							       _("Entries Auto-default"),
 							       NULL, FALSE,
 							       G_PARAM_READABLE | G_PARAM_WRITABLE));
-	g_object_class_install_property (object_class, PROP_CAN_EXPAND,
-					 g_param_spec_boolean ("can-expand",
-							       _("TRUE if expanding the form makes sense"),
+	g_object_class_install_property (object_class, PROP_CAN_VEXPAND,
+					 g_param_spec_boolean ("can-expand-v",
+							       _("TRUE if expanding the form vertically makes sense"),
 							       NULL, FALSE,
 							       G_PARAM_READABLE));
 
@@ -293,8 +298,6 @@ gdaui_basic_form_init (GdauiBasicForm * wid)
 	wid->priv->headers_sensitive = FALSE;
 	wid->priv->show_actions = FALSE;
 	wid->priv->entries_auto_default = FALSE;
-
-	wid->priv->forward_param_updates = TRUE;
 }
 
 /**
@@ -347,8 +350,11 @@ get_rid_of_set (GdaSet *paramlist, GdauiBasicForm *form)
 	/* unref the paramlist */
 	g_signal_handlers_disconnect_by_func (form->priv->set_info,
 					      G_CALLBACK (paramlist_public_data_changed_cb), form);
+
 	g_signal_handlers_disconnect_by_func (paramlist,
 					      G_CALLBACK (paramlist_param_attr_changed_cb), form);
+	g_signal_handlers_disconnect_by_func (paramlist,
+					      G_CALLBACK (paramlist_holder_type_set_cb), form);
 
 	g_object_unref (form->priv->set);
 	form->priv->set = NULL;
@@ -363,6 +369,24 @@ get_rid_of_set (GdaSet *paramlist, GdauiBasicForm *form)
 		gdaui_data_entry_set_editable (GDAUI_DATA_ENTRY (((SingleEntry*)list->data)->entry), FALSE);
 }
 
+static void
+paramlist_holder_type_set_cb (GdaSet *paramlist, GdaHolder *param,
+			      GdauiBasicForm *form)
+{
+	SingleEntry *sentry;
+
+	sentry = get_single_entry_for_holder (form, param);
+	if (sentry) {
+		create_entry_widget (sentry);
+		gdaui_data_entry_set_attributes (GDAUI_DATA_ENTRY (sentry->entry),
+						 form->priv->show_actions ? GDA_VALUE_ATTR_ACTIONS_SHOWN : 0,
+						 GDA_VALUE_ATTR_ACTIONS_SHOWN);
+		pack_entry_widget (sentry);
+		gdaui_basic_form_entry_set_visible (form, param, !sentry->hidden);
+	}
+}
+
+
 static void
 paramlist_public_data_changed_cb (GdauiSet *paramlist, GdauiBasicForm *form)
 {
@@ -423,7 +447,33 @@ paramlist_param_attr_changed_cb (GdaSet *paramlist, GdaHolder *param,
 			gdaui_basic_form_entry_set_visible (form, param, !sentry->hidden);
 		}
 		else
-			paramlist_public_data_changed_cb (form->priv->set_info, form);	
+			paramlist_public_data_changed_cb (form->priv->set_info, form);
+	}
+	else if (!strcmp (att_name, GDA_ATTRIBUTE_NAME) ||
+		 !strcmp (att_name, GDA_ATTRIBUTE_DESCRIPTION)) {
+		if (sentry) {
+			gchar *str, *title;
+			str = create_text_label_for_sentry (sentry, &title);
+			gtk_label_set_text (GTK_LABEL (sentry->label), str);
+			g_free (str);
+			g_free (sentry->label_title);
+			sentry->label_title = title;
+			
+			if (! sentry->group->group->nodes_source) {
+				g_object_get (G_OBJECT (param), "description", &title, NULL);
+				if (title && *title)
+					gtk_widget_set_tooltip_text (sentry->label, title);
+				g_free (title);
+			}
+			else {
+				title = g_object_get_data (G_OBJECT (sentry->group->group->nodes_source->data_model),
+							   "descr");
+				if (title && *title)
+					gtk_widget_set_tooltip_text (sentry->label, title);
+			}
+		}
+		else
+			paramlist_public_data_changed_cb (form->priv->set_info, form);
 	}
 }
 
@@ -501,6 +551,8 @@ gdaui_basic_form_set_property (GObject *object,
 						  G_CALLBACK (paramlist_public_data_changed_cb), form);
 				g_signal_connect (form->priv->set, "holder-attr-changed",
 						  G_CALLBACK (paramlist_param_attr_changed_cb), form);
+				g_signal_connect (form->priv->set, "holder-type-set",
+						  G_CALLBACK (paramlist_holder_type_set_cb), form);
 
 				create_entries (form);
 				pack_entries_in_table (form);
@@ -546,7 +598,7 @@ gdaui_basic_form_get_property (GObject *object,
 		case PROP_ENTRIES_AUTO_DEFAULT:
 			g_value_set_boolean (value, form->priv->entries_auto_default);
 			break;
-		case PROP_CAN_EXPAND: {
+		case PROP_CAN_VEXPAND: {
 			gboolean can_expand = FALSE;
 			GSList *list;
 			for (list = form->priv->s_entries; list; list = list->next) {
@@ -568,9 +620,12 @@ gdaui_basic_form_get_property (GObject *object,
 static void
 disconnect_single_entry_signals (SingleEntry *sentry)
 {
-	if (sentry->entry)
+	if (sentry->entry) {
 		g_signal_handler_disconnect (sentry->entry, sentry->entry_contents_modified_id);
+		g_signal_handler_disconnect (sentry->entry, sentry->entry_expand_changed_id);
+	}
 	sentry->entry_contents_modified_id = 0;
+	sentry->entry_expand_changed_id = 0;
 	if (sentry->entry)
 		g_signal_handler_disconnect (sentry->entry, sentry->entry_contents_activated_id);
 	sentry->entry_contents_activated_id = 0;
@@ -651,12 +706,66 @@ destroy_entries (GdauiBasicForm *form)
 	}
 }
 
+static gchar *
+create_text_label_for_sentry (SingleEntry *sentry, gchar **out_title)
+{
+	gchar *label = NULL;
+	g_assert (out_title);
+	if (! sentry->group->group->nodes_source) {
+		g_object_get (G_OBJECT (sentry->single_param), "name", out_title, NULL);
+		if (!*out_title)
+			*out_title = g_strdup (_("Value"));
+		label = g_strdup_printf ("%s:", *out_title);
+		/*
+		g_object_get (G_OBJECT (param), "description", &title, NULL);
+		if (title && *title)
+			gtk_widget_set_tooltip_text (sentry->label, title);
+		g_free (title);
+		*/
+
+	}
+	else {
+		gchar *label;
+		GSList *params;
+		gchar *title = NULL;
+
+		label = g_object_get_data (G_OBJECT (sentry->group->group->nodes_source->data_model), "name");
+		if (label)
+			title = g_strdup (label);
+		else {
+			GString *tstring = NULL;
+			for (params = sentry->group->group->nodes; params; params = params->next) {
+				g_object_get (G_OBJECT (GDA_SET_NODE (params->data)->holder),
+					      "name", &title, NULL);
+				if (title) {
+					if (tstring) 
+						g_string_append (tstring, ",\n");
+					else
+						tstring = g_string_new ("");
+					g_string_append (tstring, title);
+				}
+			}
+			if (tstring)
+				title = g_string_free (tstring, FALSE);
+		}
+
+		if (!title)
+			title = g_strdup (_("Value"));
+
+		label = g_strdup_printf ("%s:", title);
+		*out_title = title;
+	}
+
+	return label;
+}
+
 static void
 create_entry_widget (SingleEntry *sentry)
 {
 	GdauiSetGroup *group;
 	GtkWidget *entry;
 	gboolean editable = TRUE;
+	GValue *ref_value = NULL;
 
 	disconnect_single_entry_signals (sentry);
 	if (sentry->entry) {
@@ -670,10 +779,15 @@ create_entry_widget (SingleEntry *sentry)
 			sentry->entry_shown_id = 0;
 		}
 
+		const GValue *cvalue;
+		cvalue = gdaui_data_entry_get_reference_value (sentry->entry);
+		if (cvalue)
+			ref_value = gda_value_copy (cvalue);
 		editable = gdaui_data_entry_get_editable (sentry->entry);
 		g_object_unref ((GObject *) sentry->entry);
 		sentry->entry = NULL;
 	}
+
 	if (sentry->label) {
 		GtkWidget *parent;
 		parent = gtk_widget_get_parent (sentry->label);
@@ -703,7 +817,7 @@ create_entry_widget (SingleEntry *sentry)
 
 		param = GDA_HOLDER (GDA_SET_NODE (group->group->nodes->data)->holder);
 		sentry->single_param = param;
-			
+		
 		val = gda_holder_get_value (param);
 		default_val = gda_holder_get_default_value (param);
 		nnul = gda_holder_get_not_null (param);
@@ -730,22 +844,27 @@ create_entry_widget (SingleEntry *sentry)
 		/* set current value */
 		if (gda_holder_is_valid (param))
 			gdaui_data_entry_set_value (GDAUI_DATA_ENTRY (entry), val);
-		else {
-			GValue value;
-			memset (&value, 0, sizeof (GValue));
-			gdaui_data_entry_set_value (GDAUI_DATA_ENTRY (entry), &value);
-		}
+		else
+			gdaui_data_entry_set_value (GDAUI_DATA_ENTRY (entry), NULL);
 
-		if (!nnul ||
-		    (nnul && value &&
-		     (G_VALUE_TYPE ((GValue *) value) != GDA_TYPE_NULL)))
+		if (ref_value) {
+			gdaui_data_entry_set_reference_value (GDAUI_DATA_ENTRY (entry), ref_value);
+			gda_value_free (ref_value);
+		}
+		else if (!nnul ||
+			 (nnul && value &&
+			  (G_VALUE_TYPE ((GValue *) value) != GDA_TYPE_NULL)))
 			gdaui_data_entry_set_reference_value (GDAUI_DATA_ENTRY (entry), value);
 
 		if (default_val) {
-			gdaui_data_entry_set_value_default (GDAUI_DATA_ENTRY (entry), default_val);
+			gdaui_data_entry_set_default_value (GDAUI_DATA_ENTRY (entry), default_val);
 			gdaui_data_entry_set_attributes (GDAUI_DATA_ENTRY (entry),
 							 GDA_VALUE_ATTR_CAN_BE_DEFAULT,
 							 GDA_VALUE_ATTR_CAN_BE_DEFAULT);
+			if (gda_holder_value_is_default (param))
+				gdaui_data_entry_set_attributes (GDAUI_DATA_ENTRY (entry),
+							 GDA_VALUE_ATTR_IS_DEFAULT,
+							 GDA_VALUE_ATTR_IS_DEFAULT);
 		}
 
 		gdaui_data_entry_set_attributes (GDAUI_DATA_ENTRY (entry),
@@ -761,10 +880,7 @@ create_entry_widget (SingleEntry *sentry)
 		/* label */
 		gchar *title;
 		gchar *str;
-		g_object_get (G_OBJECT (param), "name", &title, NULL);
-		if (!title)
-			title = g_strdup (_("Value"));
-		str = g_strdup_printf ("%s:", title);
+		str = create_text_label_for_sentry (sentry, &title);
 		sentry->label = gtk_label_new (str);
 		g_free (str);
 		g_object_ref_sink (sentry->label);
@@ -780,7 +896,7 @@ create_entry_widget (SingleEntry *sentry)
 	else {
 		/* several parameters depending on the values of a GdaDataModel object */
 		GSList *plist;
-		gboolean nnul = TRUE;
+		gboolean nullok = TRUE;
 			
 		entry = gdaui_entry_combo_new (sentry->form->priv->set_info, group->source);
 
@@ -790,8 +906,8 @@ create_entry_widget (SingleEntry *sentry)
 			GdaHolder *param;
 
 			param = GDA_SET_NODE (plist->data)->holder;
-			if (!gda_holder_get_not_null (param))
-				nnul = FALSE;
+			if (gda_holder_get_not_null (param))
+				nullok = FALSE;
 
 			SignalData sd;
 			sd.holder = g_object_ref (param);
@@ -801,32 +917,15 @@ create_entry_widget (SingleEntry *sentry)
 			g_array_append_val (sentry->group_signals, sd);
 		}
 		gdaui_data_entry_set_attributes (GDAUI_DATA_ENTRY (entry),
-						 nnul ? 0 : GDA_VALUE_ATTR_CAN_BE_NULL,
+						 nullok ? GDA_VALUE_ATTR_CAN_BE_NULL : 0,
 						 GDA_VALUE_ATTR_CAN_BE_NULL);
-		sentry->not_null = nnul;
+		sentry->not_null = !nullok;
 
 		/* label */
-		/* FIXME: find a better label and tooltip and improve data entry attributes */
 		gchar *title = NULL;
 		gchar *str;
-		GSList *params;
-			
-		for (params = sentry->group->group->nodes; params; params = params->next) {
-			g_object_get (G_OBJECT (GDA_SET_NODE (params->data)->holder),
-				      "name", &title, NULL);
-			if (title)
-				break;
-		}
-			
-		if (!title) {
-			str = g_object_get_data (G_OBJECT (group->group->nodes_source->data_model),
-						 "name");
-			if (str)
-				title = g_strdup (str);
-		}
-		if (!title)
-			title = g_strdup (_("Value"));
-		str = g_strdup_printf ("%s:", title);
+
+		str = create_text_label_for_sentry (sentry, &title);
 		sentry->label = gtk_label_new (str);
 		g_free (str);
 		g_object_ref_sink (sentry->label);
@@ -854,7 +953,7 @@ create_entry_widget (SingleEntry *sentry)
 			break;
 		case GDAUI_BASIC_FORM_ENTRIES:
 			if (sentry->entry)
-				gtk_size_group_add_widget (sg->size_group, sentry->entry);
+				gtk_size_group_add_widget (sg->size_group, GTK_WIDGET (sentry->entry));
 			break;
 		default:
 			g_assert_not_reached ();
@@ -865,6 +964,9 @@ create_entry_widget (SingleEntry *sentry)
 	sentry->entry_contents_modified_id = g_signal_connect (G_OBJECT (entry), "contents-modified",
 							       G_CALLBACK (entry_contents_modified),
 							       sentry);
+	sentry->entry_expand_changed_id = g_signal_connect (sentry->entry, "expand-changed",
+							    G_CALLBACK (entry_expand_changed_cb), sentry);
+
 	sentry->entry_contents_activated_id = g_signal_connect (G_OBJECT (entry), "contents-activated",
 								G_CALLBACK (entry_contents_activated),
 								sentry->form);
@@ -889,6 +991,7 @@ create_entries (GdauiBasicForm *form)
 
 		sentry = g_new0 (SingleEntry, 1);
 		sentry->form = form;
+		sentry->forward_param_updates = TRUE;
 		form->priv->s_entries = g_slist_append (form->priv->s_entries, sentry);
 
 		sentry->group = GDAUI_SET_GROUP (list->data);
@@ -905,7 +1008,7 @@ static void
 pack_entry_widget (SingleEntry *sentry)
 {
 	gboolean expand;
-	expand = gdaui_data_entry_expand_in_layout (GDAUI_DATA_ENTRY (sentry->entry));
+	expand = gdaui_data_entry_can_expand (GDAUI_DATA_ENTRY (sentry->entry), FALSE);
 	sentry->can_expand = expand;
 
 	switch (sentry->packing_type) {
@@ -1253,13 +1356,17 @@ entry_contents_modified (GdauiDataEntry *entry, SingleEntry *sentry)
 	if (param) { /* single parameter */
 		GValue *value;
 
-		sentry->form->priv->forward_param_updates = FALSE;
+		sentry->forward_param_updates = FALSE;
 
 		/* parameter's value */
 		value = gdaui_data_entry_get_value (entry);
-		if ((!value || gda_value_is_null (value)) &&
-		    (attr & GDA_VALUE_ATTR_IS_DEFAULT))
+		if (attr & GDA_VALUE_ATTR_IS_DEFAULT)
 			gda_holder_set_value_to_default (param);
+		else if (attr & GDA_VALUE_ATTR_DATA_NON_VALID) {
+			gda_holder_force_invalid (param);
+			g_signal_emit (G_OBJECT (sentry->form), gdaui_basic_form_signals[HOLDER_CHANGED],
+				       0, param, TRUE);
+		}
 		else if (gda_holder_set_value (param, value, NULL))
 			g_signal_emit (G_OBJECT (sentry->form), gdaui_basic_form_signals[HOLDER_CHANGED],
 				       0, param, TRUE);
@@ -1274,7 +1381,7 @@ entry_contents_modified (GdauiDataEntry *entry, SingleEntry *sentry)
 						  sentry->entry_contents_modified_id);
 		}
 		gda_value_free (value);
-		sentry->form->priv->forward_param_updates = TRUE;
+		sentry->forward_param_updates = TRUE;
 	}
 	else { /* multiple parameters */
 		GSList *params;
@@ -1297,14 +1404,14 @@ entry_contents_modified (GdauiDataEntry *entry, SingleEntry *sentry)
 			 *   and emit only one signal.
 			 */
 			GdaHolder *param;
-			sentry->form->priv->forward_param_updates = FALSE;
+			sentry->forward_param_updates = FALSE;
 
 			/* parameter's value */
 			param = GDA_SET_NODE (params->data)->holder;
 			gda_holder_set_value (param, (GValue *)(list->data), NULL);
 			g_signal_emit (G_OBJECT (sentry->form), gdaui_basic_form_signals[HOLDER_CHANGED],
 				       0, param, TRUE);
-			sentry->form->priv->forward_param_updates = TRUE;
+			sentry->forward_param_updates = TRUE;
 		}
 		g_slist_free (values);
 
@@ -1339,10 +1446,16 @@ entry_contents_modified (GdauiDataEntry *entry, SingleEntry *sentry)
 	}
 }
 
+static void
+entry_expand_changed_cb (GdauiDataEntry *entry, SingleEntry *sentry)
+{
+	pack_entry_widget (sentry);
+}
+
 
 /*
  * Called when a parameter changes
- * We emit a "holder-changed" signal only if the 'form->priv->forward_param_updates' is TRUE, which means
+ * We emit a "holder-changed" signal only if the 'sentry->forward_param_updates' is TRUE, which means
  * the param change does not come from a GdauiDataEntry change.
  */
 static void
@@ -1352,7 +1465,7 @@ parameter_changed_cb (GdaHolder *param, SingleEntry *sentry)
 	GdauiDataEntry *entry;
 
 	entry = sentry->entry;
-	if (sentry->form->priv->forward_param_updates) {
+	if (sentry->forward_param_updates) {
 		gboolean param_valid;
 		gboolean default_if_invalid = FALSE;
 
@@ -1402,11 +1515,17 @@ parameter_changed_cb (GdaHolder *param, SingleEntry *sentry)
 						  sentry->entry_contents_modified_id);
 			g_signal_handler_unblock (G_OBJECT (entry),
 						  sentry->entry_contents_activated_id);
-		}
+		}		
+
+		gdaui_entry_shell_set_unknown (GDAUI_ENTRY_SHELL (entry),
+					       !gda_holder_is_valid (param));
 
 		g_signal_emit (G_OBJECT (sentry->form), gdaui_basic_form_signals[HOLDER_CHANGED], 0,
 			       param, FALSE);
 	}
+	else
+		gdaui_entry_shell_set_unknown (GDAUI_ENTRY_SHELL (entry),
+					       !gda_holder_is_valid (param));
 }
 
 /**
@@ -1457,9 +1576,9 @@ gdaui_basic_form_set_as_reference (GdauiBasicForm *form)
 			}
 
 			if (!allnull)
-				gdaui_entry_combo_set_values_orig (GDAUI_ENTRY_COMBO (sentry->entry), values);
+				gdaui_entry_combo_set_reference_values (GDAUI_ENTRY_COMBO (sentry->entry), values);
 			else
-				gdaui_entry_combo_set_values_orig (GDAUI_ENTRY_COMBO (sentry->entry), NULL);
+				gdaui_entry_combo_set_reference_values (GDAUI_ENTRY_COMBO (sentry->entry), NULL);
 
 			g_slist_free (values);
 		}
@@ -1580,7 +1699,7 @@ gdaui_basic_form_reset (GdauiBasicForm *form)
 			/* Combo entry */
 			GSList *values = NULL;
 
-			values = gdaui_entry_combo_get_values_orig (GDAUI_ENTRY_COMBO (sentry->entry));
+			values = gdaui_entry_combo_get_reference_values (GDAUI_ENTRY_COMBO (sentry->entry));
 			gdaui_entry_combo_set_values (GDAUI_ENTRY_COMBO (sentry->entry), values);
 			g_slist_free (values);
 		}
@@ -1588,7 +1707,7 @@ gdaui_basic_form_reset (GdauiBasicForm *form)
 			/* non combo entry */
 			const GValue *value;
 
-			value = gdaui_data_entry_get_original_value (GDAUI_DATA_ENTRY (sentry->entry));
+			value = gdaui_data_entry_get_reference_value (GDAUI_DATA_ENTRY (sentry->entry));
 			gdaui_data_entry_set_value (GDAUI_DATA_ENTRY (sentry->entry), value);
 		}
 	}
@@ -1599,7 +1718,7 @@ gdaui_basic_form_reset (GdauiBasicForm *form)
  * gdaui_basic_form_entry_set_visible
  * @form: a #GdauiBasicForm widget
  * @param: a #GdaHolder object
- * @show:
+ * @show: set to %TRUE to show the data entry, and to %FALSE to hide it
  *
  * Shows or hides the #GdauiDataEntry in @form which corresponds to the
  * @param parameter
@@ -1683,23 +1802,23 @@ gdaui_basic_form_entry_grab_focus (GdauiBasicForm *form, GdaHolder *param)
 /**
  * gdaui_basic_form_entry_set_editable
  * @form: a #GdauiBasicForm widget
- * @param: a #GdaHolder object; or %NULL
+ * @holder: a #GdaHolder object; or %NULL
  * @editable: %TRUE if corresponding data entry must be editable
  *
  * Sets the #GdauiDataEntry in @form which corresponds to the
- * @param parameter editable or not. If @param is %NULL, then all the parameters
+ * @holder parameter editable or not. If @holder is %NULL, then all the parameters
  * are concerned.
  *
  * Since: 4.2
  */
 void
-gdaui_basic_form_entry_set_editable (GdauiBasicForm *form, GdaHolder *param, gboolean editable)
+gdaui_basic_form_entry_set_editable (GdauiBasicForm *form, GdaHolder *holder, gboolean editable)
 {
 	g_return_if_fail (GDAUI_IS_BASIC_FORM (form));
 
-	if (param) {
+	if (holder) {
 		SingleEntry *sentry;
-		sentry = get_single_entry_for_holder (form, param);
+		sentry = get_single_entry_for_holder (form, holder);
 		if (sentry)
 			gdaui_data_entry_set_editable (sentry->entry, editable);
 	}
@@ -1917,7 +2036,7 @@ gdaui_basic_form_new_in_dialog (GdaSet *data_set, GtkWindow *parent,
 
 
 	gboolean can_expand;
-	g_object_get ((GObject*) form, "can-expand", &can_expand, NULL);
+	g_object_get ((GObject*) form, "can-expand-v", &can_expand, NULL);
 #if GTK_CHECK_VERSION(2,18,0)
 	gtk_container_set_border_width (GTK_CONTAINER (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg)))), 4);
 	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg))), form,
@@ -2022,7 +2141,8 @@ gdaui_basic_form_set_layout_from_file (GdauiBasicForm *form, const gchar *file_n
  * @form: a #GdauiBasicForm
  * @placeholder_id: the name of the requested place holder
  *
- * Retreives a pointer to a place holder widget
+ * Retreives a pointer to a place holder widget. This feature is only available if a specific
+ * layout has been defined for @form using gdaui_basic_form_set_layout_from_file().
  *
  * Returns: a pointer to the requested place holder, or %NULL if not found
  *
