@@ -1,5 +1,5 @@
 /* GDA common library
- * Copyright (C) 1998 - 2009 The GNOME Foundation.
+ * Copyright (C) 1998 - 2010 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -38,15 +38,13 @@
 #endif
 #include <libgda/sql-parser/gda-sql-statement.h>
 #include <sql-parser/gda-statement-struct-util.h>
+#include <libgda/gda-set.h>
+#include <libgda/gda-blob-op.h>
 
 #include <libgda/binreloc/gda-binreloc.h>
 
-/* we don't want to duplicate the symbols in <libgda/sqlite/keywords_hash.h>, so simply
- * declare them as external
- */
-extern const unsigned char UpperToLower[];
-#define charMap(X) UpperToLower[(unsigned char)(X)]
-extern int casecmp(const char *zLeft, const char *zRight, int N);
+#define KEYWORDS_HASH_NO_STATIC
+#include <libgda/sqlite/keywords_hash.h>
 #include "keywords_hash.c" /* this one is dynamically generated */
 
 extern gchar *gda_lang_locale;
@@ -113,7 +111,7 @@ gda_g_type_to_string (GType type)
  *   <listitem><para>"null" for GDA_TYPE_NULL</para></listitem>
  * </itemizedlist>
  *
- * Returns: the #GType represented by the given @str.
+ * Returns: the #GType represented by the given @str, or #G_TYPE_INVALID if not found
  */
 GType
 gda_g_type_from_string (const gchar *str)
@@ -207,8 +205,8 @@ gda_default_escape_string (const gchar *string)
  * gda_default_unescape_string
  * @string: string to unescape
  *
- * Do the reverse of gda_default_escape_string(): transforms any "\'" into "'" and any
- * "\\" into "\". 
+ * Do the reverse of gda_default_escape_string(): transforms any "''" into "'", any
+ * "\\" into "\" and any "\'" into "'". 
  *
  * Returns: a new unescaped string, or %NULL in an error was found in @string
  */
@@ -328,7 +326,7 @@ gda_utility_check_data_model (GdaDataModel *model, gint nbcols, ...)
  * @nb_cols: the number of columns in @cols
  * @rows: an array containing which rows of @model will be exported, or %NULL for all rows
  * @nb_rows: the number of rows in @rows
- * @use_col_ids:
+ * @use_col_ids: set to %TRUE to add column ID information
  *
  * Dump the data in a #GdaDataModel into a xmlNodePtr (as used in libxml).
  *
@@ -406,6 +404,17 @@ gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
 						if (g_value_get_string (value))
 							str = gda_value_stringify (value);	
 					}
+					else if (G_VALUE_TYPE (value) == GDA_TYPE_BLOB) {
+						/* force reading the whole blob */
+						const GdaBlob *blob = gda_value_get_blob (value);
+						if (blob) {
+							const GdaBinary *bin = &(blob->data);
+							if (blob->op && 
+							    (bin->binary_length != gda_blob_op_get_length (blob->op)))
+								gda_blob_op_read_all (blob->op, (GdaBlob*) blob);
+						}
+						str = gda_value_stringify (value);
+					}
 					else
 						str = gda_value_stringify (value);
 				}
@@ -503,7 +512,7 @@ gda_utility_data_model_find_column_description (GdaDataSelect *model, const gcha
 
 /**
  * gda_utility_holder_load_attributes
- * @holder:
+ * @holder: a #GdaHolder
  * @node: an xmlNodePtr with a &lt;parameter&gt; tag
  * @sources: a list of #GdaDataModel
  * @error: a place to store errors, or %NULL
@@ -544,6 +553,16 @@ gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *
 	else
 		gda_holder_set_not_null (holder, FALSE);
 	
+	str = xmlGetProp (node, BAD_CAST "plugin");
+	if (str) {
+		GValue *value;
+#define GDAUI_ATTRIBUTE_PLUGIN "__gdaui_attr_plugin"
+                value = gda_value_new_from_string ((gchar*) str, G_TYPE_STRING);
+		gda_holder_set_attribute_static (holder, GDAUI_ATTRIBUTE_PLUGIN, value);
+		gda_value_free (value);
+		xmlFree (str);
+	}
+
 	str = xmlGetProp (node, BAD_CAST "source");
 	if (str) 
 		g_object_set_data_full (G_OBJECT (holder), "source", str, xmlFree);
@@ -589,6 +608,7 @@ gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *
 				}
 			}
 		}
+		g_free (source);
 	}
 
 	/* specified value */
@@ -666,7 +686,7 @@ gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *
 #define GDA_PARAM_ENCODE_TOKEN "__gda"
 /**
  * gda_text_to_alphanum
- * @text:
+ * @text: the text to convert
  *
  * The "encoding" consists in replacing non
  * alphanumeric character with the string "__gdaXX" where XX is the hex. representation
@@ -1003,9 +1023,11 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 									     require_pk, error);
 		if (!ust->cond) {
 			retval = FALSE;
-			goto cleanup;
+			*update_stmt = NULL;
+			update_stmt = NULL; /* don't try anymore to build UPDATE statement */
 		}
-		GDA_SQL_ANY_PART (ust->cond)->parent = GDA_SQL_ANY_PART (ust);
+		else
+			GDA_SQL_ANY_PART (ust->cond)->parent = GDA_SQL_ANY_PART (ust);
 	}
         
 	if (delete_stmt) {
@@ -1015,14 +1037,19 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 
 		dst->table = gda_sql_table_new (GDA_SQL_ANY_PART (dst));
 		dst->table->table_name = g_strdup (tmp);
-		dst->cond = gda_compute_unique_table_row_condition_with_cnc (cnc, stsel, 
+		if (update_stmt && ust->cond)
+			dst->cond = gda_sql_expr_copy (ust->cond);
+		else
+			dst->cond = gda_compute_unique_table_row_condition_with_cnc (cnc, stsel, 
 									     GDA_META_TABLE (target->validity_meta_object),
 									     require_pk, error);
 		if (!dst->cond) {
 			retval = FALSE;
-			goto cleanup;
+			*delete_stmt = NULL;
+			delete_stmt = NULL; /* don't try anymore to build DELETE statement */
 		}
-		GDA_SQL_ANY_PART (dst->cond)->parent = GDA_SQL_ANY_PART (dst);
+		else
+			GDA_SQL_ANY_PART (dst->cond)->parent = GDA_SQL_ANY_PART (dst);
 	}
 	g_free (tmp);
 
@@ -1094,10 +1121,11 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
 				     "%s", _("Could not compute any field to insert into"));
 			retval = FALSE;
-			goto cleanup;
 		}
-		ist->values_list = g_slist_append (NULL, insert_values_list);
-		ret_insert = g_object_new (GDA_TYPE_STATEMENT, "structure", sql_ist, NULL);
+		else {
+			ist->values_list = g_slist_append (NULL, insert_values_list);
+			ret_insert = g_object_new (GDA_TYPE_STATEMENT, "structure", sql_ist, NULL);
+		}
 	}
 	if (update_stmt)
 		ret_update = g_object_new (GDA_TYPE_STATEMENT, "structure", sql_ust, NULL);
@@ -1131,7 +1159,7 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
  * however that this GdaSqlStatement does not select anything (ie it would be rendered as "SELECT FROM ... WHERE ...")
  * and before being usable, one needs to add some fields to actually select.
  *
- * Returns; a new #GdaStatement if no error occurred, or %NULL otherwise
+ * Returns: a new #GdaStatement if no error occurred, or %NULL otherwise
  */
 GdaSqlStatement *
 gda_compute_select_statement_from_update (GdaStatement *update_stmt, GError **error)
@@ -1175,6 +1203,235 @@ gda_compute_select_statement_from_update (GdaStatement *update_stmt, GError **er
 	return sel_stmt;
 }
 
+static gboolean stmt_rewrite_insert_remove (GdaSqlStatementInsert *ins, GdaSet *params, GError **error);
+static gboolean stmt_rewrite_insert_default_keyword (GdaSqlStatementInsert *ins, GdaSet *params, GError **error);
+static gboolean stmt_rewrite_update_default_keyword (GdaSqlStatementUpdate *upd, GdaSet *params, GError **error);
+
+
+/**
+ * gda_statement_rewrite_for_default_values
+ * @stmt: a #GdaStatement object
+ * @params: a #GdaSet containing the variable's values to be bound when executing @stmt
+ * @remove: set to %TRUE if DEFAULT fields are removed, of %FALSE if the "DEFAULT" keyword is used
+ * @error: a place to store errors, or %NULL
+ *
+ * Rewrites @stmt and creates a new #GdaSqlStatement where all the variables which are to a DEFAULT value
+ * (as returned by gda_holder_value_is_default()) are either removed from the statement (if @remove
+ * is %TRUE) or replaced by the "DEFAULT" keyword (if @remove is %FALSE).
+ *
+ * This function is only usefull for database providers' implementations which have to deal with default
+ * values when executing statements, and is only relevant in the case of INSERT or UPDATE statements
+ * (in the latter case an error is returned if @remove is %TRUE).
+ *
+ * For example the <programlisting><![CDATA[INSERT INTO mytable (id, name) VALUES (23, ##name::string)]]></programlisting>
+ * is re-written into <programlisting><![CDATA[INSERT INTO mytable (id, name) VALUES (23, DEFAULT)]]></programlisting>
+ * if @remove is %FALSE and into <programlisting><![CDATA[INSERT INTO mytable (id) VALUES (23)]]></programlisting>
+ * if @remove is %TRUE.
+ *
+ * Returns: a new #GdaSqlStatement, or %NULL if an error occurred
+ *
+ * Since: 4.2
+ */
+GdaSqlStatement *
+gda_statement_rewrite_for_default_values (GdaStatement *stmt, GdaSet *params, gboolean remove, GError **error)
+{
+	GdaSqlStatement *sqlst;
+	GdaSqlStatementType type;
+	gboolean ok = FALSE;
+	g_return_val_if_fail (GDA_IS_STATEMENT (stmt), NULL);
+	g_return_val_if_fail (GDA_IS_SET (params), NULL);
+	type = gda_statement_get_statement_type (stmt);
+
+	g_object_get (stmt, "structure", &sqlst, NULL);
+	if (! gda_sql_statement_check_structure (sqlst, error)) {
+		gda_sql_statement_free (sqlst);
+		return NULL;		
+	}
+
+	switch (type) {
+	case GDA_SQL_STATEMENT_INSERT:
+		if (remove)
+			ok = stmt_rewrite_insert_remove ((GdaSqlStatementInsert*) sqlst->contents,
+							 params, error);
+		else
+			ok = stmt_rewrite_insert_default_keyword ((GdaSqlStatementInsert*) sqlst->contents,
+								  params, error);
+		break;
+	case GDA_SQL_STATEMENT_UPDATE:
+		if (remove)
+			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+				     GDA_SERVER_PROVIDER_DEFAULT_VALUE_HANDLING_ERROR,
+				     _("Can't rewrite UPDATE statement to handle default values"));
+		else
+			ok = stmt_rewrite_update_default_keyword ((GdaSqlStatementUpdate*) sqlst->contents,
+								  params, error);
+		break;
+	default:
+		g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+			     GDA_SERVER_PROVIDER_DEFAULT_VALUE_HANDLING_ERROR,
+			     "Can't rewrite statement is not INSERT or UPDATE");
+		break;
+	}
+
+	if (ok)
+		return sqlst;
+	else {
+		gda_sql_statement_free (sqlst);
+		return NULL;
+	}
+}
+
+/*
+ * Modifies @ins
+ * Returns: TRUE if rewrite is Ok
+ */
+static gboolean
+stmt_rewrite_insert_remove (GdaSqlStatementInsert *ins, GdaSet *params, GError **error)
+{
+	if (!ins->values_list)
+		/* nothing to do */
+		return TRUE;
+
+	if (ins->values_list->next) {
+		TO_IMPLEMENT;
+		g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+			     GDA_SERVER_PROVIDER_DEFAULT_VALUE_HANDLING_ERROR,
+			     "Not yet implemented");
+		return FALSE;
+	}
+
+	GSList *fields, *values;
+	for (fields = ins->fields_list, values = (GSList*) ins->values_list->data;
+	     fields && values; ){
+		GdaHolder *h;
+		GdaSqlExpr *expr = (GdaSqlExpr*) values->data;
+		if (! expr->param_spec || ! expr->param_spec->is_param) {
+			fields = fields->next;
+			values = values->next;
+			continue;
+		}
+		h = gda_set_get_holder (params, expr->param_spec->name);
+		if (!h) {
+			gchar *str;
+			str = g_strdup_printf (_("Missing parameter '%s' to execute query"),
+					       expr->param_spec->name);
+			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+				     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR,
+				     "%s", str);
+			g_free (str);
+			return FALSE;
+		}
+
+		if (gda_holder_value_is_default (h)) {
+			GSList *tmp;
+			
+			gda_sql_field_free ((GdaSqlField*) fields->data);
+			tmp = fields->next;
+			ins->fields_list = g_slist_delete_link (ins->fields_list, fields);
+			fields = tmp;
+			
+			gda_sql_expr_free (expr);
+			tmp = values->next;
+			ins->values_list->data = g_slist_delete_link ((GSList*) ins->values_list->data,
+								      values);
+			values = tmp;
+		}
+		else {
+			fields = fields->next;
+			values = values->next;
+		}
+	}
+
+	if (! ins->values_list->data) {
+		g_slist_free (ins->values_list);
+		ins->values_list = NULL;
+	}
+
+	return TRUE;
+}
+
+/*
+ * Modifies @ins
+ * Returns: TRUE if rewrite is Ok
+ */
+static gboolean
+stmt_rewrite_insert_default_keyword (GdaSqlStatementInsert *ins, GdaSet *params, GError **error)
+{
+	GSList *llist;
+	for (llist = ins->values_list; llist; llist = llist->next) {
+		GSList *values;
+		for (values = (GSList*) llist->data;
+		     values;
+		     values = values->next){
+			GdaHolder *h;
+			GdaSqlExpr *expr = (GdaSqlExpr*) values->data;
+			if (! expr->param_spec || ! expr->param_spec->is_param)
+				continue;
+			h = gda_set_get_holder (params, expr->param_spec->name);
+			if (!h) {
+				gchar *str;
+				str = g_strdup_printf (_("Missing parameter '%s' to execute query"),
+						       expr->param_spec->name);
+				g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+					     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR,
+					     "%s", str);
+				g_free (str);
+				return FALSE;
+			}
+			
+			if (gda_holder_value_is_default (h)) {
+				GdaSqlExpr *nexpr;
+				nexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (ins));
+				g_value_set_string ((nexpr->value = gda_value_new (G_TYPE_STRING)),
+						    "DEFAULT");
+				gda_sql_expr_free ((GdaSqlExpr*) values->data);
+				values->data = nexpr;
+			}
+		}
+	}
+
+	if (! ins->values_list->data) {
+		g_slist_free (ins->values_list);
+		ins->values_list = NULL;
+	}
+
+	return TRUE;
+}
+
+static gboolean
+stmt_rewrite_update_default_keyword (GdaSqlStatementUpdate *upd, GdaSet *params, GError **error)
+{
+	GSList *values;
+	for (values = upd->expr_list; values; values = values->next) {
+		GdaHolder *h;
+		GdaSqlExpr *expr = (GdaSqlExpr*) values->data;
+		if (! expr->param_spec || ! expr->param_spec->is_param)
+			continue;
+		h = gda_set_get_holder (params, expr->param_spec->name);
+		if (!h) {
+			gchar *str;
+			str = g_strdup_printf (_("Missing parameter '%s' to execute query"),
+					       expr->param_spec->name);
+			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+				     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR,
+				     "%s", str);
+			g_free (str);
+			return FALSE;
+		}
+		
+		if (gda_holder_value_is_default (h)) {
+			GdaSqlExpr *nexpr;
+			nexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (upd));
+			g_value_set_string ((nexpr->value = gda_value_new (G_TYPE_STRING)),
+					    "DEFAULT");
+			gda_sql_expr_free ((GdaSqlExpr*) values->data);
+			values->data = nexpr;
+		}
+	}
+
+	return TRUE;
+}
+
 /**
  * gda_identifier_hash
  * @id: an identifier string
@@ -1307,6 +1564,12 @@ prepare_sql_identifier_for_compare (gchar *str)
 	}
 }
 
+static gint
+cmp_func (gconstpointer a, gconstpointer b)
+{
+	return g_strcmp0 (*((gchar**) a), *((gchar**) b));
+}
+
 /**
  * gda_completion_list_get
  * @cnc: a #GdaConnection object
@@ -1330,6 +1593,8 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 	if (!cnc) 
 		return NULL;
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	if (!sql || !(*sql))
+		return NULL;
 	if (end < start)
 		return NULL;
 
@@ -1482,7 +1747,7 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 						
 						g_object_unref (m2);
 					}
-					free (str);
+					g_free (str);
 				}
 			}
 		}
@@ -1516,6 +1781,23 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 	g_free (text);
 	if (compl) {
 		if (compl->len >= 1) {
+			/* sort */
+			gint i;
+			g_array_sort (compl, cmp_func);
+
+			/* remove duplicates if any */
+			for (i = 1; i < compl->len; ) {
+				gchar *current, *before;
+				current = g_array_index (compl, gchar*, i);
+				before = g_array_index (compl, gchar*, i - 1);
+				if (!strcmp (current, before)) {
+					g_free (current);
+					g_array_remove_index (compl, i);
+				}
+				else
+					i++;
+			}
+
 			gchar **ptr;
 			ptr = (gchar**) compl->data;
 			g_array_free (compl, FALSE);
@@ -1561,7 +1843,7 @@ concat_ident (const char *prefix, const gchar *ident)
  *
  * For example the <![CDATA["test.\"ATable\""]]> string will result in the array: <![CDATA[{"test", "\"ATable\"", NULL}]]>
  *
- * Returns: a new array of strings, or NULL (use g_strfreev() to free the returned array)
+ * Returns: a new %NULL-terminated array of strings, or NULL (use g_strfreev() to free the returned array)
  */
 gchar **
 gda_sql_identifier_split (const gchar *id)
@@ -1786,6 +2068,9 @@ gda_sql_identifier_quote (const gchar *id, GdaConnection *cnc, GdaServerProvider
 			prov = gda_connection_get_provider (cnc);
 	}
 
+	if ((*id == '*') && (! id [1]))
+	    return g_strdup (id);
+
 	if (prov && PROV_CLASS (prov)->identifier_quote)
 		return PROV_CLASS (prov)->identifier_quote (prov, cnc, id,
 							    for_meta_store, force_quotes);
@@ -2107,6 +2392,26 @@ gda_dsn_split (const gchar *string, gchar **out_dsn, gchar **out_username, gchar
  * in @string, the various parts are strings
  * which are expected to be encoded using an RFC 1738 compliant encoding. If they are specified, 
  * the returned provider, username and password strings are correctly decoded.
+ *
+ * For example all the following connection strings:
+ * <programlisting><![CDATA[
+PostgreSQL://meme:pass@DB_NAME=mydb;HOST=server
+PostgreSQL://meme@DB_NAME=mydb;HOST=server;PASSWORD=pass
+PostgreSQL://meme@DB_NAME=mydb;PASSWORD=pass;HOST=server
+PostgreSQL://meme@PASSWORD=pass;DB_NAME=mydb;HOST=server
+PostgreSQL://DB_NAME=mydb;HOST=server;USERNAME=meme;PASSWORD=pass
+PostgreSQL://DB_NAME=mydb;HOST=server;PASSWORD=pass;USERNAME=meme
+PostgreSQL://DB_NAME=mydb;USERNAME=meme;PASSWORD=pass;HOST=server
+PostgreSQL://PASSWORD=pass;USERNAME=meme;DB_NAME=mydb;HOST=server
+PostgreSQL://:pass@USERNAME=meme;DB_NAME=mydb;HOST=server
+PostgreSQL://:pass@DB_NAME=mydb;HOST=server;USERNAME=meme]]></programlisting>
+ *
+ * will return the following new strings (double quotes added here to delimit strings):
+ * <programlisting><![CDATA[
+out_cnc_params: "DB_NAME=mydb;HOST=server"
+out_provider: "PostgreSQL"
+out_username: "meme"
+out_password: "pass"]]></programlisting>
  */
 void
 gda_connection_string_split (const gchar *string, gchar **out_cnc_params, gchar **out_provider, 
@@ -2155,6 +2460,46 @@ gda_connection_string_split (const gchar *string, gchar **out_cnc_params, gchar
 	if (!*out_cnc_params)
 		*out_cnc_params = g_strdup (ap);
 
+	if (*out_cnc_params) {
+		gchar *pos;
+
+		pos = g_strrstr (*out_cnc_params, "USERNAME=");
+		if (pos) {
+			for (ptr = pos + 9; ptr && *ptr != '\0' && *ptr != ';'; ptr++);
+			if (ptr != pos + 9)
+				*out_username = g_strndup (pos + 9, ptr - (pos + 9));
+
+			if (*ptr)
+				g_memmove (pos, ptr + 1, strlen (ptr));
+			else
+				*pos = 0;
+			gchar *tmp;
+			gint len;
+			tmp = *out_cnc_params;
+			len = strlen (tmp) - 1;
+			if (tmp [len] == ';')
+				tmp [len] = 0;
+		}
+
+		pos = g_strrstr (*out_cnc_params, "PASSWORD=");
+		if (pos) {
+			for (ptr = pos + 9; ptr && *ptr != '\0' && *ptr != ';'; ptr++);
+			if (ptr != pos + 9)
+				*out_password = g_strndup (pos + 9, ptr - (pos + 9));
+
+			if (*ptr)
+				g_memmove (pos, ptr + 1, strlen (ptr));
+			else
+				*pos = 0;
+			gchar *tmp;
+			gint len;
+			tmp = *out_cnc_params;
+			len = strlen (tmp) - 1;
+			if (tmp [len] == ';')
+				tmp [len] = 0;
+		}
+	}
+
 	/* RFC 1738 decode provider, username and password strings */
 	gda_rfc1738_decode (*out_provider);
 	gda_rfc1738_decode (*out_username);
