@@ -1,5 +1,5 @@
 /* GDA MDB Provider
- * Copyright (C) 1998 - 2007 The GNOME Foundation
+ * Copyright (C) 1998 - 2008 The GNOME Foundation
  *
  * AUTHORS:
  *         Rodrigo Moya <rodrigo@gnome-db.org>
@@ -21,17 +21,39 @@
  */
 
 #include <glib/gi18n-lib.h>
-#include "gda-mdb.h"
+#include <gmodule.h>
 #include <libgda/gda-server-provider-extra.h>
+#include <libgda/binreloc/gda-binreloc.h>
+#include "gda-mdb.h"
+#include "gda-mdb-provider.h"
 
 static gchar *module_path = NULL;
-MdbSQL *mdb_SQL = NULL;
 
 const gchar       *plugin_get_name (void);
 const gchar       *plugin_get_description (void);
 gchar             *plugin_get_dsn_spec (void);
 GdaServerProvider *plugin_create_provider (void);
 
+/*
+ * Functions executed when calling g_module_open() and g_module_close()
+ */
+const gchar *
+g_module_check_init (GModule *module)
+{
+        /*g_module_make_resident (module);*/
+        return NULL;
+}
+
+void
+g_module_unload (GModule *module)
+{
+        g_free (module_path);
+        module_path = NULL;
+}
+
+/*
+ * Normal plugin functions 
+ */
 void
 plugin_init (const gchar *real_path)
 {
@@ -42,7 +64,7 @@ plugin_init (const gchar *real_path)
 const gchar *
 plugin_get_name (void)
 {
-	return "MS Access";
+	return MDB_PROVIDER_NAME;
 }
 
 const gchar *
@@ -54,7 +76,23 @@ plugin_get_description (void)
 gchar *
 plugin_get_dsn_spec (void)
 {
-	return gda_server_provider_load_file_contents (module_path, LIBGDA_DATA_DIR, "mdb_specs_dsn.xml");
+	gchar *ret, *dir;
+
+        dir = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, NULL);
+        ret = gda_server_provider_load_file_contents (module_path, dir, "mdb_specs_dsn.xml");
+        g_free (dir);
+        return ret;
+}
+
+gchar *
+plugin_get_auth_spec (void)
+{
+#define AUTH "<?xml version=\"1.0\"?>" \
+             "<data-set-spec>" \
+             "  <parameters/>" \
+             "</data-set-spec>"
+
+        return g_strdup (AUTH);
 }
 
 GdaServerProvider *
@@ -63,28 +101,6 @@ plugin_create_provider (void)
 	GdaServerProvider *prov;
 
         prov = gda_mdb_provider_new ();
-        g_object_set_data (prov, "GDA_PROVIDER_DIR", module_path);
+        g_object_set_data (G_OBJECT (prov), "GDA_PROVIDER_DIR", module_path);
         return prov;
 }
-
-/*
- * GModule functions
- */
-
-const gchar *
-g_module_check_init (void)
-{
-	if (!mdb_SQL)
-		mdb_SQL = mdb_sql_init ();
-
-	return NULL;
-}
-
-void
-g_module_unload (void)
-{
-	if (mdb_SQL) {
-		mdb_sql_exit (mdb_SQL);
-		mdb_SQL = NULL;
-	}
-}
