@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2007 - 2008 The GNOME Foundation.
+ * Copyright (C) 2007 - 2009 The GNOME Foundation.
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -19,17 +19,92 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#undef GDA_DISABLE_DEPRECATED
 #include "command-exec.h"
 #include <glib/gi18n-lib.h>
 #include <string.h>
 #include "tools-input.h"
-#ifdef HAVE_READLINE_HISTORY_H
+#ifdef HAVE_HISTORY
 #include <readline/history.h>
 #endif
-
+#include <sql-parser/gda-statement-struct-util.h>
 
 /*
- * command_exec_result_free
+ *  gda_internal_command_arg_remove_quotes
+ *
+ * If @str has simple or double quotes, remove those quotes, otherwise does nothing.
+ *
+ * Returns: @str
+ */
+gchar *
+gda_internal_command_arg_remove_quotes (gchar *str)
+{
+	glong total;
+        gchar *ptr;
+        glong offset = 0;
+        char delim;
+
+        if (!str)
+                return NULL;
+        delim = *str;
+        if ((delim != '\'') && (delim != '"'))
+                return str;
+
+
+        total = strlen (str);
+        if (str[total-1] == delim) {
+                /* string is correctly terminated by a double quote */
+                g_memmove (str, str+1, total-2);
+                total -=2;
+        }
+        else {
+                /* string is _not_ correctly terminated by a double quote */
+                g_memmove (str, str+1, total-1);
+                total -=1;
+        }
+        str[total] = 0;
+
+        ptr = (gchar *) str;
+        while (offset < total) {
+                /* we accept the "''" as a synonym of "\'" */
+                if (*ptr == delim) {
+                        if (*(ptr+1) == delim) {
+                                g_memmove (ptr+1, ptr+2, total - offset);
+                                offset += 2;
+                        }
+                        else {
+                                *str = 0;
+                                return str;
+                        }
+                }
+                if (*ptr == '\\') {
+                        if (*(ptr+1) == '\\') {
+                                g_memmove (ptr+1, ptr+2, total - offset);
+                                offset += 2;
+                        }
+                        else {
+                                if (*(ptr+1) == delim) {
+                                        *ptr = delim;
+                                        g_memmove (ptr+1, ptr+2, total - offset);
+                                        offset += 2;
+                                }
+                                else {
+                                        *str = 0;
+                                        return str;
+                                }
+			}
+                }
+                else
+                        offset ++;
+
+                ptr++;
+        }
+
+        return str;
+}
+
+/*
+ * gda_internal_command_exec_result_free
  *
  * Clears the memory associated with @res
  */
@@ -66,166 +141,110 @@ gda_internal_command_exec_result_free (GdaInternalCommandResult *res)
 	g_free (res);
 }
 
-static GdaInternalCommand *
-find_command (GdaInternalCommandsList *commands_list, const gchar *command_str, gboolean *command_complete)
+/*
+ * Small tokenizer
+ */
+#define T_ILLEGAL                         1
+#define T_SPACE                           2
+#define T_TEXTUAL                         3
+#define T_WORD                            4
+#define T_STRING                          5
+static int
+getToken (const char *z, gint *tok_type)
 {
-	GdaInternalCommand *command = NULL;
-	GSList *list;
-	gint length;
+	*tok_type = T_SPACE;
+	int i, c;
 
-	if (!command_str || ((*command_str != '\\') && (*command_str != '.')))
-		return NULL;
-
-	length = strlen (command_str + 1);
-	for (list = commands_list->name_ordered; list; list = list->next) {
-		command = (GdaInternalCommand*) list->data;
-		if (!strncmp (command->name, command_str + 1, MIN (length, strlen (command->name)))) {
-			gint l;
-			gchar *ptr;
-			for (ptr = command->name, l = 0; *ptr && (*ptr != ' '); ptr++, l++);
-				
-			if (length == l)
-				break;
-			else
-				command = NULL;
-		}
-		else
-			command = NULL;
+	switch (*z) {
+	case ' ': case '\t': case '\n': case '\f': case '\r': {
+		for (i=1; g_ascii_isspace (z[i]); i++){}
+		*tok_type = T_SPACE;
+		return i;
 	}
-
-	/* FIXME */
-	if (command_complete)
-		*command_complete = TRUE;
-
-	return command;
-}
-
-static gint
-commands_compare_name (GdaInternalCommand *a, GdaInternalCommand *b)
-{
-	gint cmp, alength, blength;
-	if (!a->name || !b->name) {
-		g_warning (_("Invalid unnamed command"));
-		if (!a->name) {
-			if (b->name)
-				return 1;
-			else
-				return 0;
-		}
-		else
-			return -1;
-	}
-	alength = strlen (a->name);
-	blength = strlen (b->name);
-	cmp = strncmp (a->name, b->name, MIN (alength, blength));
-	if (cmp == 0) 
-		return blength - alength;
-	else
-		return cmp;
-}
-
-static gint
-commands_compare_group (GdaInternalCommand *a, GdaInternalCommand *b)
-{
-	if (!a->group) {
-		if (b->group)
-			return 1;
-		else
-			return 0;
-	}
-	else {
-		if (b->group) {
-			gint cmp = strcmp (a->group, b->group);
-			if (cmp)
-				return cmp;
-			else 
-				return commands_compare_name (a, b);
-		}
-		else
-			return -1;
+	case '\'':
+        case '"': {
+                char delim = z[0];
+                for (i = 1; (c = z[i]) != 0; i++) {
+                        if (c == delim) {
+                                if (z[i+1] == delim)
+                                        i++;
+                                else
+                                        break;
+                        }
+                        else if (c == '\\') {
+                                if (z[i+1] == delim)
+                                        i++;
+                        }
+                }
+                if (c) {
+                        if (delim == '"')
+                                *tok_type = T_TEXTUAL;
+                        else
+                                *tok_type = T_STRING;
+			return i+1;
+                }
+                else {
+                        *tok_type = T_ILLEGAL;
+                        return 0;
+                }
+                break;
+        }
+	default: 
+		for (i=0; z[i] && !g_ascii_isspace (z[i]); i++){}
+		*tok_type = T_WORD;
+		return i;
 	}
 }
 
 /* default function to split arguments */
-static gchar **
+gchar **
 default_gda_internal_commandargs_func (const gchar *string)
 {
-	gchar **array, **ptr;
+	gchar **array = NULL;
+	GSList *parsed_list = NULL;
+	gchar *ptr;
+	gint nparsed, token_type;
 
-	array = g_strsplit (string, " ", -1);
-	for (ptr = array; *ptr; ptr++)
-		g_strchug (*ptr);
+	ptr = (gchar *) string; 
+	nparsed = getToken (ptr, &token_type);
+	while (nparsed > 0) {
+		gint local_nparsed = nparsed;
+		gchar *local_ptr = ptr;
+		switch (token_type) {
+		case T_SPACE:
+		case T_ILLEGAL:
+			break;
+		case T_TEXTUAL:
+		case T_STRING:
+		default: {
+			gchar hold;
+			hold = local_ptr[local_nparsed];
+			local_ptr[local_nparsed] = 0;
+			parsed_list = g_slist_append (parsed_list, g_strdup (local_ptr));
+			local_ptr[local_nparsed] = hold;
+			break;
+		}
+		}
+		ptr += nparsed;
+		nparsed = getToken (ptr, &token_type);
+	}
+	
+	if (parsed_list) {
+		GSList *list;
+		gint i;
+		array = g_new0 (gchar*, g_slist_length (parsed_list) + 1);
+		for (i = 0, list = parsed_list; list; i++, list = list->next) {
+			array [i] = (gchar *) list->data;
+			/*g_print ("array [%d] = %s\n", i, array [i]);*/
+		}
+		g_slist_free (parsed_list);
+	}
 
 	return array;
 }
 
-
-/*
- * gda_internal_command_execute
- *
- * Executes a command starting by \ such as:
- */
 GdaInternalCommandResult *
-gda_internal_command_execute (GdaInternalCommandsList *commands_list,
-			      GdaConnection *cnc, const gchar *command_str, GError **error)
-{
-	GdaInternalCommand *command;
-	gboolean command_complete;
-	gchar **args;
-	GdaInternalCommandResult *res = NULL;
-
-	g_return_val_if_fail (commands_list, NULL);
-	if (!commands_list->name_ordered) {
-		GSList *list;
-
-		for (list = commands_list->commands; list; list = list->next) {
-			commands_list->name_ordered = 
-				g_slist_insert_sorted (commands_list->name_ordered, list->data,
-						       (GCompareFunc) commands_compare_name);
-			commands_list->group_ordered = 
-				g_slist_insert_sorted (commands_list->group_ordered, list->data,
-						       (GCompareFunc) commands_compare_group);
-		}
-	}
-
-	args = g_strsplit (command_str, " ", 2);
-	command = find_command (commands_list, args[0], &command_complete);
-	g_strfreev (args);
-	args = NULL;
-	if (!command) {
-		g_set_error (error, 0, 0,
-			     _("Unknown internal command"));
-		goto cleanup;
-	}
-
-	if (!command->command_func) {
-		g_set_error (error, 0, 0,
-			     _("Internal command not correctly defined"));
-		goto cleanup;
-	}
-
-	if (!command_complete) {
-		g_set_error (error, 0, 0,
-			     _("Incomplete internal command"));
-		goto cleanup;
-	}
-
-	if (command->arguments_delimiter_func)
-		args = command->arguments_delimiter_func (command_str);
-	else
-		args = default_gda_internal_commandargs_func (command_str);
-	res = command->command_func (cnc, (const gchar **) &(args[1]), 
-				     error, command->user_data);
-	
- cleanup:
-	if (args)
-		g_strfreev (args);
-
-	return res;
-}
-
-GdaInternalCommandResult *
-gda_internal_command_help (GdaConnection *cnc, const gchar **args, 
+gda_internal_command_help (SqlConsole *console, GdaConnection *cnc, const gchar **args, 
 			   GError **error,
 			   GdaInternalCommandsList *clist)
 {
@@ -240,6 +259,10 @@ gda_internal_command_help (GdaConnection *cnc, const gchar **args,
 
 	for (list = clist->group_ordered; list; list = list->next) {
 		GdaInternalCommand *command = (GdaInternalCommand*) list->data;
+
+		if (console && command->limit_to_main)
+			continue;
+
 		if (!current_group || strcmp (current_group, command->group)) {
 			current_group = command->group;
 			if (list != clist->group_ordered)
@@ -266,15 +289,21 @@ gda_internal_command_help (GdaConnection *cnc, const gchar **args,
 }
 
 GdaInternalCommandResult *
-gda_internal_command_history (GdaConnection *cnc, const gchar **args, GError **error, gpointer data)
+gda_internal_command_history (SqlConsole *console, GdaConnection *cnc, const gchar **args, GError **error, gpointer data)
 {
 	GdaInternalCommandResult *res;
 
 	res = g_new0 (GdaInternalCommandResult, 1);
 	res->type = GDA_INTERNAL_COMMAND_RESULT_TXT;
 
+	if (console) {
+		res->type = GDA_INTERNAL_COMMAND_RESULT_EMPTY;
+		TO_IMPLEMENT;
+		return res;
+	}
+
 	GString *string;
-#ifdef HAVE_READLINE_HISTORY_H
+#ifdef HAVE_HISTORY
 	if (args[0]) {
 		if (!save_history (args[0], error)) {
 			g_free (res);
@@ -305,19 +334,33 @@ gda_internal_command_history (GdaConnection *cnc, const gchar **args, GError **e
 }
 
 GdaInternalCommandResult *
-gda_internal_command_dict_sync (GdaConnection *cnc, const gchar **args, GError **error, gpointer data)
+gda_internal_command_dict_sync (SqlConsole *console, GdaConnection *cnc, const gchar **args, GError **error, gpointer data)
 {
 	GdaInternalCommandResult *res;
 
 	if (!cnc) {
-		g_set_error (error, 0, 0, _("No current connection"));
+		g_set_error (error, 0, 0, "%s", _("No current connection"));
 		return NULL;
 	}
 
 	res = g_new0 (GdaInternalCommandResult, 1);
 	res->type = GDA_INTERNAL_COMMAND_RESULT_EMPTY;
 
-	if (!gda_connection_update_meta_store (cnc, NULL, error)) {
+	if (args[0] && *args[0]) {
+		GdaMetaContext context;
+		memset (&context, 0, sizeof (context));
+		if (*args[0] == '_')
+			context.table_name = (gchar*) args[0];
+		else
+			context.table_name = g_strdup_printf ("_%s", args[0]);
+		if (!gda_connection_update_meta_store (cnc, &context, error)) {
+			g_free (res);
+			res = NULL;
+		}
+		if (*args[0] != '_')
+			g_free (context.table_name);
+	}
+	else if (!gda_connection_update_meta_store (cnc, NULL, error)) {
 		g_free (res);
 		res = NULL;
 	}
@@ -326,13 +369,13 @@ gda_internal_command_dict_sync (GdaConnection *cnc, const gchar **args, GError *
 }
 
 GdaInternalCommandResult *
-gda_internal_command_list_tables (GdaConnection *cnc, const gchar **args, GError **error, gpointer data)
+gda_internal_command_list_tables (SqlConsole *console, GdaConnection *cnc, const gchar **args, GError **error, gpointer data)
 {
 	GdaInternalCommandResult *res;
 	GdaDataModel *model;
 
 	if (!cnc) {
-		g_set_error (error, 0, 0, _("No current connection"));
+		g_set_error (error, 0, 0, "%s", _("No current connection"));
 		return NULL;
 	}
 
@@ -340,18 +383,19 @@ gda_internal_command_list_tables (GdaConnection *cnc, const gchar **args, GError
 		GValue *v;
 		const gchar *sql = "SELECT table_schema AS Schema, table_name AS Name, table_type as Type, "
 			"table_owner as Owner, table_comments as Description "
-			"FROM _tables WHERE table_name=##tname::string AND "
-			"table_type LIKE '%TABLE%' AND table_short_name = table_name "
+			"FROM _tables WHERE table_short_name=##tname::string AND "
+			"table_type LIKE '%TABLE%' "
 			"ORDER BY table_schema, table_name";
 
-		g_value_set_string (v = gda_value_new (G_TYPE_STRING), args[0]);
+		gchar *tmp = gda_sql_identifier_remove_quotes (g_strdup (args[0]));
+		g_value_take_string (v = gda_value_new (G_TYPE_STRING), tmp);
 		model = gda_meta_store_extract (gda_connection_get_meta_store (cnc), sql, error, "tname", v, NULL);
 		gda_value_free (v);
 	}
 	else {
 		const gchar *sql = "SELECT table_schema AS Schema, table_name AS Name, table_type as Type, "
 			"table_owner as Owner, table_comments as Description "
-			"FROM _tables WHERE table_type LIKE '%TABLE%' AND table_short_name = table_name "
+			"FROM _tables WHERE table_type LIKE '%TABLE%' "
 			"ORDER BY table_schema, table_name";
 		model = gda_meta_store_extract (gda_connection_get_meta_store (cnc), sql, error);
 	}
@@ -368,13 +412,13 @@ gda_internal_command_list_tables (GdaConnection *cnc, const gchar **args, GError
 }
 
 GdaInternalCommandResult *
-gda_internal_command_list_views (GdaConnection *cnc, const gchar **args, GError **error, gpointer data)
+gda_internal_command_list_views (SqlConsole *console, GdaConnection *cnc, const gchar **args, GError **error, gpointer data)
 {
 	GdaInternalCommandResult *res;
 	GdaDataModel *model;
 
 	if (!cnc) {
-		g_set_error (error, 0, 0, _("No current connection"));
+		g_set_error (error, 0, 0, "%s", _("No current connection"));
 		return NULL;
 	}
 
@@ -382,8 +426,8 @@ gda_internal_command_list_views (GdaConnection *cnc, const gchar **args, GError
 		GValue *v;
 		const gchar *sql = "SELECT table_schema AS Schema, table_name AS Name, table_type as Type, "
 			"table_owner as Owner, table_comments as Description "
-			"FROM _tables WHERE table_name=##tname::string AND "
-			"table_type = 'VIEW' AND table_short_name = table_name "
+			"FROM _tables WHERE table_short_name=##tname::string AND "
+			"table_type = 'VIEW' "
 			"ORDER BY table_schema, table_name";
 
 		g_value_set_string (v = gda_value_new (G_TYPE_STRING), args[0]);
@@ -393,7 +437,7 @@ gda_internal_command_list_views (GdaConnection *cnc, const gchar **args, GError
 	else {
 		const gchar *sql = "SELECT table_schema AS Schema, table_name AS Name, table_type as Type, "
 			"table_owner as Owner, table_comments as Description "
-			"FROM _tables WHERE table_type='VIEW' AND table_short_name = table_name "
+			"FROM _tables WHERE table_type='VIEW' "
 			"ORDER BY table_schema, table_name";
 		model = gda_meta_store_extract (gda_connection_get_meta_store (cnc), sql, error);
 	}
@@ -410,13 +454,13 @@ gda_internal_command_list_views (GdaConnection *cnc, const gchar **args, GError
 }
 
 GdaInternalCommandResult *
-gda_internal_command_list_schemas (GdaConnection *cnc, const gchar **args, GError **error, gpointer data)
+gda_internal_command_list_schemas (SqlConsole *console, GdaConnection *cnc, const gchar **args, GError **error, gpointer data)
 {
 	GdaInternalCommandResult *res;
 	GdaDataModel *model;
 
 	if (!cnc) {
-		g_set_error (error, 0, 0, _("No current connection"));
+		g_set_error (error, 0, 0, "%s", _("No current connection"));
 		return NULL;
 	}
 
@@ -449,67 +493,6 @@ gda_internal_command_list_schemas (GdaConnection *cnc, const gchar **args, GErro
 	return res;
 }
 
-GdaInternalCommandResult *
-gda_internal_command_list_queries (GdaConnection *cnc, const gchar **args,
-				   GError **error, gpointer data)
-{
-	GdaInternalCommandResult *res = NULL;
-	GdaDataModel *model;
-	const gchar *qname = NULL;
-	gboolean with_sql_def = FALSE;
-
-	if (!cnc) {
-		g_set_error (error, 0, 0, _("No current connection"));
-		return NULL;
-	}
-
-	if (args[0] && *args[0]) {
-		if (!strcmp (args[0], "+"))
-			with_sql_def = TRUE;
-		else {
-			qname = args[0];
-			if (args[1] && (*args[1] == '+'))
-				with_sql_def = TRUE;
-		}
-	}
-
-	if (with_sql_def)
-		model = gda_data_model_array_new_with_g_types (5,
-							       G_TYPE_STRING,
-							       G_TYPE_STRING,
-							       G_TYPE_STRING,
-							       G_TYPE_STRING,
-							       G_TYPE_STRING);
-	else
-		model = gda_data_model_array_new_with_g_types (4,
-							       G_TYPE_STRING,
-							       G_TYPE_STRING,
-							       G_TYPE_STRING,
-							       G_TYPE_STRING);
-	gda_data_model_set_column_title (model, 0, _("Name"));
-	gda_data_model_set_column_title (model, 1, _("Type"));
-	gda_data_model_set_column_title (model, 2, _("Description"));
-	gda_data_model_set_column_title (model, 3, _("Parameters"));
-	if (with_sql_def)
-		gda_data_model_set_column_title (model, 4, _("SQL"));
-	g_object_set_data (G_OBJECT (model), "name", _("List of queries"));
-
-	if (qname) {
-		TO_IMPLEMENT;
-	}
-	else {
-		
-
-		TO_IMPLEMENT;
-	}
-
-	res = g_new0 (GdaInternalCommandResult, 1);
-	res->type = GDA_INTERNAL_COMMAND_RESULT_DATA_MODEL;
-	res->u.model = model;
-
-	return res;
-}
-
 GdaMetaStruct *
 gda_internal_command_build_meta_struct (GdaConnection *cnc, const gchar **args, GError **error)
 {
@@ -517,32 +500,24 @@ gda_internal_command_build_meta_struct (GdaConnection *cnc, const gchar **args,
 	gint index;
 	const gchar *arg;
 	GdaMetaStore *store;
+	GSList *objlist;
 
 	store = gda_connection_get_meta_store (cnc);
-	mstruct = gda_meta_struct_new (GDA_META_STRUCT_FEATURE_ALL);
+	mstruct = gda_meta_struct_new (store, GDA_META_STRUCT_FEATURE_ALL);
 
 	if (!args[0]) {
-		/* use all tables or views */
-		GdaDataModel *model;
-		gint i, nrows;
-		const gchar *sql = "SELECT t.table_catalog, t.table_schema, t.table_name, v.table_name FROM _tables as t LEFT JOIN _views as v ON (t.table_catalog=v.table_catalog AND t.table_schema=v.table_schema AND t.table_name=v.table_name) WHERE table_short_name != table_full_name";
-		model = gda_meta_store_extract (store, sql, error, NULL);
-		if (!model)
-			return NULL;
-		nrows = gda_data_model_get_n_rows (model);
-		for (i = 0; i < nrows; i++) {
-			const GValue *detv;
-			detv = gda_data_model_get_value_at (model, 3, i);
-			if (! gda_meta_struct_complement (mstruct, store, 
-							  detv && !gda_value_is_null (detv) && 
-							  g_value_get_string (detv) && *g_value_get_string (detv) ? 
-							  GDA_META_DB_VIEW : GDA_META_DB_TABLE,
-							  gda_data_model_get_value_at (model, 0, i),
-							  gda_data_model_get_value_at (model, 1, i),
-							  gda_data_model_get_value_at (model, 2, i), error)) 
+		GSList *list;
+		/* use all tables or views visible by default */
+		if (!gda_meta_struct_complement_default (mstruct, error))
+			goto onerror;
+		list = gda_meta_struct_get_all_db_objects (mstruct);
+		if (!list) {
+			/* use all tables or views visible or not by default */
+			if (!gda_meta_struct_complement_all (mstruct, error))
 				goto onerror;
 		}
-		g_object_unref (model);
+		else
+			g_slist_free (list);
 	}
 
 	for (index = 0, arg = args[0]; arg; index++, arg = args[index]) {
@@ -552,76 +527,41 @@ gda_internal_command_build_meta_struct (GdaConnection *cnc, const gchar **args,
 		/* see if we have the form <schema_name>.*, to list all the objects in a given schema */
 		if (g_str_has_suffix (arg, ".*") && (*arg != '.')) {
 			gchar *str;
-			GdaDataModel *model;
-			gint i, nrows;
 
 			str = g_strdup (arg);
 			str[strlen (str) - 2] = 0;
 			g_value_take_string (v, str);
-			const gchar *sql = "SELECT t.table_catalog, t.table_schema, t.table_name, v.table_name FROM _tables as t LEFT JOIN _views as v ON (t.table_catalog=v.table_catalog AND t.table_schema=v.table_schema AND t.table_name=v.table_name) WHERE t.table_schema = ##ts::string";
-			model = gda_meta_store_extract (store, sql, error, "ts", v, NULL);
-			if (!model)
-				return NULL;
-			nrows = gda_data_model_get_n_rows (model);
-			for (i = 0; i < nrows; i++) {
-				const GValue *detv;
-				detv = gda_data_model_get_value_at (model, 3, i);
-				if (! gda_meta_struct_complement (mstruct, store, 
-								  detv && !gda_value_is_null (detv) && 
-								  g_value_get_string (detv) && *g_value_get_string (detv) ? 
-								  GDA_META_DB_VIEW : GDA_META_DB_TABLE,
-								  gda_data_model_get_value_at (model, 0, i),
-								  gda_data_model_get_value_at (model, 1, i),
-								  gda_data_model_get_value_at (model, 2, i), error)) 
-					goto onerror;
-			}
-			g_object_unref (model);
+
+			if (!gda_meta_struct_complement_schema (mstruct, NULL, v, error))
+				goto onerror;
 		}
 		else {
 			/* try to find it as a table or view */
-			if (!gda_meta_struct_complement (mstruct, store, GDA_META_DB_UNKNOWN, NULL, NULL, v, NULL)) {
-				if (g_str_has_suffix (arg, "=") && (*arg != '=')) {
-					GdaMetaDbObject *dbo;
-					gchar *str;
-					str = g_strdup (arg);
-					str[strlen (str) - 1] = 0;
-					g_value_take_string (v, str);
-					dbo = gda_meta_struct_complement (mstruct, store, GDA_META_DB_TABLE, 
-									  NULL, NULL, v, NULL);
-					if (!dbo)
-						dbo = gda_meta_struct_complement (mstruct, store, GDA_META_DB_VIEW, 
-										  NULL, NULL, v, NULL);
-					if (dbo && dbo->depend_list) {
-						GSList *list, *dep_list;
-						GValue *catalog, *schema, *name;
-
-						dep_list = g_slist_copy (dbo->depend_list);
-						for (list = dep_list; list; list = list->next) {
-							dbo = GDA_META_DB_OBJECT (list->data);
-							g_value_set_string ((catalog = gda_value_new (G_TYPE_STRING)), 
-									    dbo->obj_catalog);
-							g_value_set_string ((schema = gda_value_new (G_TYPE_STRING)), 
-									    dbo->obj_schema);
-							g_value_set_string ((name = gda_value_new (G_TYPE_STRING)), 
-									    dbo->obj_name);
-							gda_meta_struct_complement (mstruct, store, dbo->obj_type,
-										    catalog, schema, name, NULL);
-							gda_value_free (catalog);
-							gda_value_free (schema);
-							gda_value_free (name);
-						}
-						g_slist_free (dep_list);
-					}
-				}
+			if (g_str_has_suffix (arg, "=") && (*arg != '=')) {
+				GdaMetaDbObject *dbo;
+				gchar *str;
+				str = g_strdup (arg);
+				str[strlen (str) - 1] = 0;
+				g_value_take_string (v, str);
+				dbo = gda_meta_struct_complement (mstruct, GDA_META_DB_UNKNOWN, 
+								  NULL, NULL, v, error);
+				if (dbo)
+					gda_meta_struct_complement_depend (mstruct, dbo, error);
+				else
+					goto onerror;
 			}
+			else if (!gda_meta_struct_complement (mstruct, GDA_META_DB_UNKNOWN, NULL, NULL, v, error))
+				goto onerror;
 		}
 	}
 
-	if (!mstruct->db_objects) {
-		g_set_error (error, 0, 0,
+	objlist = gda_meta_struct_get_all_db_objects (mstruct);
+	if (!objlist) {
+		g_set_error (error, 0, 0, "%s", 
 			     _("No object found"));
 		goto onerror;
 	}
+	g_slist_free (objlist);
 	gda_meta_struct_sort_db_objects (mstruct, GDA_META_SORT_ALHAPETICAL, NULL);
 	return mstruct;
 
@@ -630,25 +570,49 @@ gda_internal_command_build_meta_struct (GdaConnection *cnc, const gchar **args,
 	return NULL;
 }
 
+static void
+meta_table_column_foreach_attribute_func (const gchar *att_name, const GValue *value, GString **string)
+{
+	if (!strcmp (att_name, GDA_ATTRIBUTE_AUTO_INCREMENT) && 
+	    (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN) && 
+	    g_value_get_boolean (value)) {
+		if (*string) {
+			g_string_append (*string, ", ");
+			g_string_append (*string, _("Auto increment"));
+		}
+		else
+			*string = g_string_new (_("Auto increment"));
+	}
+}
+
 GdaInternalCommandResult *
-gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
+gda_internal_command_detail (SqlConsole *console, GdaConnection *cnc, const gchar **args,
 			     GError **error, gpointer data)
 {
 	GdaInternalCommandResult *res;
 	GdaDataModel *model;
 
 	if (!cnc) {
-		g_set_error (error, 0, 0, _("No current connection"));
+		g_set_error (error, 0, 0, "%s", _("No current connection"));
 		return NULL;
 	}
 
 	if (!args[0] || !*args[0]) {
+		/* FIXME: include indexes and sequences when they are present in the information schema */
 		/* displays all tables, views, indexes and sequences which are "directly visible" */
 		const gchar *sql = "SELECT table_schema AS Schema, table_name AS Name, table_type as Type, "
 			"table_owner as Owner FROM _tables WHERE table_short_name = table_name "
 			"ORDER BY table_schema, table_name";
-		/* FIXME: include indexes and sequences when they are present in the information schema */
 		model = gda_meta_store_extract (gda_connection_get_meta_store (cnc), sql, error, NULL);
+
+		/* if no row, then return all the objects from all the schemas */
+		if (model && (gda_data_model_get_n_rows (model) == 0)) {
+			g_object_unref (model);
+			sql = "SELECT table_schema AS Schema, table_name AS Name, table_type as Type, "
+			"table_owner as Owner FROM _tables "
+			"ORDER BY table_schema, table_name";
+			model = gda_meta_store_extract (gda_connection_get_meta_store (cnc), sql, error, NULL);
+		}
 		res = g_new0 (GdaInternalCommandResult, 1);
 		res->type = GDA_INTERNAL_COMMAND_RESULT_DATA_MODEL;
 		res->u.model = model;
@@ -656,14 +620,15 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 	}
 
 	GdaMetaStruct *mstruct;
-	GSList *dbo_list;
+	GSList *dbo_list, *tmplist;
 	mstruct = gda_internal_command_build_meta_struct (cnc, args, error);
 	if (!mstruct)
 		return NULL;
-
+	
 	/* compute the number of known database objects */
 	gint nb_objects = 0;
-	for (dbo_list = mstruct->db_objects; dbo_list; dbo_list = dbo_list->next) {
+	tmplist = gda_meta_struct_get_all_db_objects (mstruct);
+	for (dbo_list = tmplist; dbo_list; dbo_list = dbo_list->next) {
 		GdaMetaDbObject *dbo = GDA_META_DB_OBJECT (dbo_list->data);
 		if (dbo->obj_type != GDA_META_DB_UNKNOWN)
 			nb_objects++;
@@ -676,7 +641,7 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 		gda_data_model_set_column_title (model, 1, _("Name"));
 		gda_data_model_set_column_title (model, 2, _("Type"));
 		gda_data_model_set_column_title (model, 3, _("Owner"));
-		for (dbo_list = mstruct->db_objects; dbo_list; dbo_list = dbo_list->next) {
+		for (dbo_list = tmplist; dbo_list; dbo_list = dbo_list->next) {
 			GdaMetaDbObject *dbo = GDA_META_DB_OBJECT (dbo_list->data);
 			GList *values = NULL;
 			GValue *val;
@@ -692,7 +657,7 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 			val = gda_value_new (G_TYPE_STRING);
 			switch (dbo->obj_type) {
 			case GDA_META_DB_TABLE:
-				g_value_set_string (val, "BASE TABLE");
+				g_value_set_string (val, "TABLE");
 				break;
 			case GDA_META_DB_VIEW:
 				g_value_set_string (val, "VIEW");
@@ -714,10 +679,12 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 		res = g_new0 (GdaInternalCommandResult, 1);
 		res->type = GDA_INTERNAL_COMMAND_RESULT_DATA_MODEL;
 		res->u.model = model;
+		g_slist_free (tmplist);
 		return res;
 	}
 	else if (nb_objects == 0) {
-		g_set_error (error, 0, 0, _("No object found"));
+		g_set_error (error, 0, 0, "%s", _("No object found"));
+		g_slist_free (tmplist);
 		return NULL;
 	}
 
@@ -729,7 +696,7 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 	res->u.multiple_results = NULL;
 	GdaMetaDbObject *dbo;
 
-	for (dbo_list = mstruct->db_objects; dbo_list; dbo_list = dbo_list->next) {
+	for (dbo_list = tmplist; dbo_list; dbo_list = dbo_list->next) {
 		dbo = GDA_META_DB_OBJECT (dbo_list->data);
 		if (dbo->obj_type == GDA_META_DB_UNKNOWN)
 			dbo = NULL;
@@ -737,17 +704,19 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 			break;
 	}
 	g_assert (dbo);
+	g_slist_free (tmplist);
 
 	if ((dbo->obj_type == GDA_META_DB_VIEW) || (dbo->obj_type == GDA_META_DB_TABLE)) {
 		GdaInternalCommandResult *subres;
-		GdaMetaTable *mt = GDA_META_DB_OBJECT_GET_TABLE (dbo);
+		GdaMetaTable *mt = GDA_META_TABLE (dbo);
 		GSList *list;
 
-		model = gda_data_model_array_new (4);
+		model = gda_data_model_array_new (5);
 		gda_data_model_set_column_title (model, 0, _("Column"));
 		gda_data_model_set_column_title (model, 1, _("Type"));
 		gda_data_model_set_column_title (model, 2, _("Nullable"));
 		gda_data_model_set_column_title (model, 3, _("Default"));
+		gda_data_model_set_column_title (model, 4, _("Extra"));
 		if (dbo->obj_type == GDA_META_DB_VIEW)
 			g_object_set_data_full (G_OBJECT (model), "name", 
 						g_strdup_printf (_("List of columns for view '%s'"), 
@@ -760,7 +729,8 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 			GdaMetaTableColumn *tcol = GDA_META_TABLE_COLUMN (list->data);
 			GList *values = NULL;
 			GValue *val;
-			
+			GString *string = NULL;
+
 			g_value_set_string ((val = gda_value_new (G_TYPE_STRING)), tcol->column_name);
 			values = g_list_append (values, val);
 			g_value_set_string ((val = gda_value_new (G_TYPE_STRING)), tcol->column_type);
@@ -769,6 +739,17 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 			values = g_list_append (values, val);
 			g_value_set_string ((val = gda_value_new (G_TYPE_STRING)), tcol->default_value);
 			values = g_list_append (values, val);
+
+			gda_meta_table_column_foreach_attribute (tcol, 
+					      (GdaAttributesManagerFunc) meta_table_column_foreach_attribute_func, &string);
+			if (string) {
+				g_value_take_string ((val = gda_value_new (G_TYPE_STRING)), string->str);
+				g_string_free (string, FALSE);
+			}
+			else
+				val = gda_value_new_null ();
+			values = g_list_append (values, val);
+
 			gda_data_model_append_values (model, values, NULL);
 			g_list_foreach (values, (GFunc) gda_value_free, NULL);
 			g_list_free (values);
@@ -781,7 +762,7 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 		
 		if (dbo->obj_type == GDA_META_DB_VIEW) {
 			/* VIEW specific */
-			GdaMetaView *mv = GDA_META_DB_OBJECT_GET_VIEW (dbo);
+			GdaMetaView *mv = GDA_META_VIEW (dbo);
 			
 			subres = g_new0 (GdaInternalCommandResult, 1);
 			subres->type = GDA_INTERNAL_COMMAND_RESULT_TXT;
@@ -807,14 +788,24 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 			for (i = 0; i < nrows; i++) {
 				GString *string = NULL;
 				const GValue *cvalue;
-				const gchar *str;
+				const gchar *str = NULL;
 
-				cvalue = gda_data_model_get_value_at (model, 0, i);
+				cvalue = gda_data_model_get_value_at (model, 0, i, error);
+				if (!cvalue) {
+					gda_internal_command_exec_result_free (res);
+					res = NULL;
+					goto out;
+				}
 				str = g_value_get_string (cvalue);
 				if (*str == 'P') {
 					/* primary key */
 					GdaDataModel *cols;
-					cvalue = gda_data_model_get_value_at (model, 1, i);
+					cvalue = gda_data_model_get_value_at (model, 1, i, error);
+					if (!cvalue) {
+						gda_internal_command_exec_result_free (res);
+						res = NULL;
+						goto out;
+					}
 					string = g_string_new (_("Primary key"));
 					g_string_append_printf (string, " '%s'", g_value_get_string (cvalue));
 					str = "SELECT column_name, ordinal_position "
@@ -834,8 +825,15 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 							for (j = 0; j < cnrows; j++) {
 								if (j > 0)
 									g_string_append (string, ", ");
-								g_string_append (string, 
-										 g_value_get_string (gda_data_model_get_value_at (cols, 0, j)));
+								cvalue = gda_data_model_get_value_at (cols, 0, j, error);
+								if (!cvalue) {
+									gda_internal_command_exec_result_free (res);
+									res = NULL;
+									g_object_unref (cols);
+									g_string_free (string, TRUE);
+									goto out;
+								}
+								g_string_append (string, g_value_get_string (cvalue));
 							}
 							g_string_append_c (string, ')');
 						}
@@ -845,12 +843,17 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 				else if (*str == 'F') {
 					/* foreign key */
 					GdaDataModel *cols;
-					cvalue = gda_data_model_get_value_at (model, 1, i);
+					cvalue = gda_data_model_get_value_at (model, 1, i, error);
+					if (!cvalue) {
+						gda_internal_command_exec_result_free (res);
+						res = NULL;
+						goto out;
+					}
 					string = g_string_new (_("Foreign key"));
 					g_string_append_printf (string, " '%s'", g_value_get_string (cvalue));
 
 					str = "SELECT ref_table_schema, ref_table_name, fk_column, ref_column "
-						"FROM _detailled_fk WHERE fk_table_catalog = ##tc::string "
+						"FROM _detailed_fk WHERE fk_table_catalog = ##tc::string "
 						"AND fk_table_schema = ##ts::string AND fk_table_name = ##tname::string AND "
 						"fk_constraint_name = ##cname::string "
 						"ORDER BY ordinal_position";
@@ -866,23 +869,51 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 							for (j = 0; j < cnrows; j++) {
 								if (j > 0)
 									g_string_append (string, ", ");
-								g_string_append (string, 
-										 g_value_get_string (gda_data_model_get_value_at (cols, 2, j)));
+								cvalue = gda_data_model_get_value_at (cols, 2, j, error);
+								if (!cvalue) {
+									gda_internal_command_exec_result_free (res);
+									res = NULL;
+									g_object_unref (cols);
+									g_string_free (string, TRUE);
+									goto out;
+								}
+								g_string_append (string, g_value_get_string (cvalue));
 							}
 							g_string_append (string, ") references ");
 
 							const GValue *v1, *v2;
 							
-							v1 = gda_data_model_get_value_at (cols, 0, 0);
-							v2 = gda_data_model_get_value_at (cols, 1, 0);
+							v1 = gda_data_model_get_value_at (cols, 0, 0, error);
+							if (!v1) {
+								gda_internal_command_exec_result_free (res);
+								res = NULL;
+								g_object_unref (cols);
+								g_string_free (string, TRUE);
+								goto out;
+							}
+							v2 = gda_data_model_get_value_at (cols, 1, 0, error);
+							if (!v2) {
+								gda_internal_command_exec_result_free (res);
+								res = NULL;
+								g_object_unref (cols);
+								g_string_free (string, TRUE);
+								goto out;
+							}
 							g_string_append_printf (string, " %s.%s (", 
 										g_value_get_string (v1),
 										g_value_get_string (v2));
 							for (j = 0; j < cnrows; j++) {
 								if (j > 0)
 									g_string_append (string, ", ");
-								g_string_append (string, 
-										 g_value_get_string (gda_data_model_get_value_at (cols, 2, j)));
+								cvalue = gda_data_model_get_value_at (cols, 3, j, error);
+								if (!cvalue) {
+									gda_internal_command_exec_result_free (res);
+									res = NULL;
+									g_object_unref (cols);
+									g_string_free (string, TRUE);
+									goto out;
+								}
+								g_string_append (string, g_value_get_string (cvalue));
 							}
 							g_string_append_c (string, ')');
 						}
@@ -892,7 +923,12 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 				else if (*str == 'U') {
 					/* Unique constraint */
 					GdaDataModel *cols;
-					cvalue = gda_data_model_get_value_at (model, 1, i);
+					cvalue = gda_data_model_get_value_at (model, 1, i, error);
+					if (!cvalue) {
+						gda_internal_command_exec_result_free (res);
+						res = NULL;
+						goto out;
+					}
 					string = g_string_new (_("Unique"));
 					g_string_append_printf (string, " '%s'", g_value_get_string (cvalue));
 					str = "SELECT column_name, ordinal_position "
@@ -912,8 +948,15 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 							for (j = 0; j < cnrows; j++) {
 								if (j > 0)
 									g_string_append (string, ", ");
-								g_string_append (string, 
-										 g_value_get_string (gda_data_model_get_value_at (cols, 0, j)));
+								cvalue = gda_data_model_get_value_at (cols, 0, j, error);
+								if (!cvalue) {
+									gda_internal_command_exec_result_free (res);
+									res = NULL;
+									g_object_unref (cols);
+									g_string_free (string, TRUE);
+									goto out;
+								}
+								g_string_append (string, g_value_get_string (cvalue));
 							}
 							g_string_append_c (string, ')');
 						}
@@ -940,6 +983,7 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 	else 
 		TO_IMPLEMENT;
 
+ out:
 	g_object_unref (mstruct);
 	return res;
 }
